# bash base library version 0.3.8
# (C) Copyright  Heinz Deinhart 2015-2018
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This is a bash library, it has to be sourced not called.
#
# bash-base provides a number of functions and a base setup of settings
# to support clean coding style in bash.
#
# Internal global veriables are prefixed with _BASH_BASE_, internal
# function with _bash_base_

[ "$0" != "$BASH_SOURCE" ] || {
	echo "$0: this script must be sourced, not called" >&2
	exit 1
}

# bb_logf <message> [..]
# logs message file name in $LOG_FILE if non zero
function bb_logf() {
    local now="$(date +'%Y%m%d %H:%M.%S')"
    if [ "${LOG_FILE+isset}" == "isset" ]
    then
	echo "$now ${0##*/}[${BASHPID}]: $*" \
	    >> "$LOG_FILE" || true
    fi
}

# bb_log <message> [..]
# logs message to stderr and if define to file name in $LOG_FILE
function bb_log() {
    local now="$(date +'%Y%m%d %H:%M.%S')"
    echo "$now ${0##*/}[${BASHPID}]: $*" >&2 || true
    if [ "${LOG_FILE+isset}" == "isset" ]
    then
	echo "$now ${0##*/}[${BASHPID}]: $*" \
	    >> "$LOG_FILE" || true
    fi
}

# will be removed in the future
function log() {
    bb_log  "$@" "<<bash-base: WARNING log is deprecated, use bb_log>>"
}

# bb_log_debug <level> <message> [..]
# logs message if $LOG_LEVEL is set and it is level <= $LOG_LEVEL
function bb_log_debug() {
    (( $# > 1 )) || bb_fatal "log_debug call with $# instead of 2+ args"
    local LVL="$1"
    shift

    if [ "${LOG_LEVEL+isset}" == "isset" ]
    then
	if (( "$LOG_LEVEL" < "$1" ))
	then
	    bb_log "debug<$LVL$> $*"
	fi
    fi
}

# will be removed in the future
function log_debug() {
    bb_log_debug "$@" \
	"<<bash-base: WARNING log_debug is deprecated, use bb_log_debug>>"
}

# bb_quit [<exit-code> [<log message>]]
# the safe way to clean exit a script using bash_base
function bb_quit() {
    _bash_base_call_exit_hooks
    _bb_traps_disable

    if (( $# < 2 ))
    then
	exit "${1-0}"
    fi

    local retval="$1"
    shift
    bb_log "quitting: $*"
    exit "$retval"
}

# bb_exec <cmd> [arg [..]]
# execs cmd with args if specified
function bb_exec() {
    _bash_base_call_exit_hooks
    _bb_traps_disable

    exec "$@"
}

# bb_fatal <message> [..]
# end program execution after logging message prepended with 'fatal error'
# since it produces some lines of output it is meant for internal errors
# only, if you want to abort your script with a message use bb_quit
function bb_fatal() {
    bb_log "fatal error: $*, stack trace follows:"
    _bash_base_log_stacktrace >&2
    bb_quit 1
}

# will be removed in the future
function fatal() {
    bb_fatal "$@" "<<bash-base: WARNING fatal is deprecated, use bb_fatal>>"
}

# bash_stacktrace
function _bash_base_log_stacktrace() {
    local i
    for ((i=1; i < (${#FUNCNAME[@]} - 1); i++))
    do
        bb_log "${1-}${BASH_SOURCE[$i+1]}:${BASH_LINENO[$i]} ${FUNCNAME[$i]}"
    done
}

function _bash_base_err_exit_trap() {
    local status=$?
    local traptype="$1"

    case "$traptype" in
	ERR|EXIT)
	    bb_log "unexpected exit/error near '$BASH_COMMAND'," \
		"status=$status, trap=$1, stack trace follows:"
	    _bash_base_log_stacktrace
	    bb_quit "$status"
	    ;;
	*)
	    bb_fatal "_bash_base_err_exit_trap: internal error," \
		"unknown trap type: $traptype"
    esac
}

# _bb_traps_setup
# sets up traps for ERR and EXIT to handle unexpected errors and exits
function _bb_traps_setup() {
    trap '_bash_base_err_exit_trap "ERR"' ERR
    trap '_bash_base_err_exit_trap "EXIT"' EXIT
}

# bb_traps_setup
# will be removed in the future
function bb_traps_setup() {
    bb_log "WARNING: bash-base: bb_traps_setup is deprecated and does nothing"
}

# _bb_traps_disable
# disables traps
function _bb_traps_disable() {
    trap - ERR
    trap - EXIT
}

# bb_traps_disable
# will be removed in the future
function bb_traps_disable() {
    bb_log "WARNING: bash-base: bb_traps_disable is deprecated and does nothing"
}

# bb_version
# returns the version of bash base
function bb_version() {
    echo "0.3.8"
}

# _bash_base_call_exit_hooks
# calls all exit hooks sorted by hooknames
function _bash_base_call_exit_hooks() {
    local hookname

    # check if any hooks are registered
    (( ${#_BASH_BASE_EXIT_HOOKS[@]} > 0 )) || return 0

    for hookname in "${!_BASH_BASE_EXIT_HOOKS[@]}"
    do
	"${_BASH_BASE_EXIT_HOOKS[$hookname]}" || \
	    bb_log "warning exit hook '$hookname' failed with exit code $?"
    done
}

# bb_register_exit_hook [ <hook-name> ] <function-name>
# registers a function to be called on exit, it is not possible to 
# pass arguments to that function
# hooks will be called sorted by hooknames
# note: the default hook-name is the function-name
function bb_register_exit_hook() {
    (( $# == 1 || $# == 2 )) || \
	bb_fatal "bb_register_exit_hook called with $# args insted of 1 or 2"

    local hookname="$1" hookfun
    if (( $# == 2 ))
    then
	hookfun="$2"
    else
	hookfun="$hookname"
    fi

    _BASH_BASE_EXIT_HOOKS["$hookname"]="$hookfun"
}

# bb_unregister_exit_hook <hookn-ame>
# removes a registered exit hook
function bb_unregister_exit_hook() {
    (( $# == 1 )) || \
	bb_fatal "bb_unregister_exit_hook called with $# args insted of 1"

    local hookname="$1"

    unset _BASH_BASE_EXIT_HOOKS["$hookname"]
}

# bb_require_libs <name> [..]
# sources file for libs specified
# if name is no absolute or local path it tries:
# /usr/libexec/<name>.bash
# /usr/libexec/<name>/<name>.bash
# errors:
#  1 = specified absolute/local file not fount
#  2 = specified file not found at standard path
#  3 = error while sourcing
function bb_require_libs() {
    local l

    while (( $# > 0 ))
    do
	l="$1"
	shift

	if [[ "$l" =~ ^\.?\.?/ ]] # is path absolute or local?
	then
	    if [ -r "$l" ]
	    then
		source "$l" || {
		    bb_log_debug 1 "bb_require_libs: error sourcing: $l"
		    return 3
		}
	    else
		bb_log_debug 1 "bb_require_libs: no such file: $l"
		return 1
	    fi
	else
	    if [ -r "/usr/libexec/$l.bash" ]
	    then
		source "/usr/libexec/$l.bash" || {
		    bb_log_debug 1 "bb_require_libs: " \
			"error sourcing /usr/libexec/$l.bash"
		    return 3
		}
		continue
	    fi
	    if [ -r "/usr/libexec/$l/$l.bash" ]
	    then
		source "/usr/libexec/$l/$l.bash" || {
		    bb_log_debug 1 "bb_require_libs: " \
			"error sourcing /usr/libexec/$l/$l.bash"
		    return 3
		}
		continue
	    else
		bb_log_debug 1 "bb_require_libs: no such lib found: $l"
		return 2
	    fi
	fi
    done
}

function _bash_base_main() {
    set -o errexit
    set -o pipefail
    set -o nounset
    set -o errtrace

    declare -g -A _BASH_BASE_EXIT_HOOKS=()

    _bb_traps_setup
}

_bash_base_main
