<!-- -*- mode: markdown; mode: flyspell -*- -->

# bash-base library

A library that provides a set of base functionality like logging and
error handling. To use it just source it at the very beginning of your
bash script like that:

```bash
source /usr/libexec/bash-base.bash || {
    echo "$0: fatal error: failed to source /usr/libexec/bash-base.bash" >&2
    exit 1
}
```

## API Docu

### Bash Base Version

```bash
# bb_version
# returns the version of bash base
```

### Error Handling

```bash
# bb_fatal <message> [..]
# end program execution after logging message prepended with 'fatal error'
```

```bash
# bb_register_exit_hook [ <hookname> ] <function-name>
# registers a function to be called on exit, it is not possible to 
# pass arguments to that function
# hooks will be called sorted by hooknames
# note: the default hook-name is the function-name
```

```bash
# bb_unregister_exit_hook <hookname>
# removes a registered exit hook
```

### Messaging

### Logging

Functions to append log message to files, standard IO and syslog.

```bash
# bb_log_file_add <filename>|stdout|stderr
# add file or stdout/stderr to list of log destinations
```

```bash
# bb_log_file_remove <filename>|stdout|stderr
# remove file from stdout/stderr to list of log destinations
```

```bash
# bb_log_file_clear
# clear list of log destinations
```

```bash
# bb_log <message> [..]
# logs message to stderr
```

```bash
# bb_log_debug <level> <message> [..]
# logs message if $LOG_LEVEL is set and it is level <= $LOG_LEVEL
```

### Library loading

```bash
# bb_require_libs <name> [..]
# sources file for libs specified, it tries the following files
# /usr/libexec/<name>.bash
# /usr/libexec/<name>/<name>.bash
# will fail if any lib fails to load
```

## Example Code

### Hello World Script

```bash
#!/bin/bash

# source bash base library
source /usr/libexec/bash-base.bash || {
	echo "$0: fatal error:" \
		"failed to source /usr/libexec/bash-base.bash" >&2
	exit 1
}

# always put your code into some function, e.g. main
function main() {
    bb_log "hello world"
}

main "$@"

# always use bb_quit to quit, it cares about traps and hooks
bb_quit
```

### Various Error Cases

```bash
#!/bin/bash

source /usr/libexec/bash-base.bash || {
	echo "$0: fatal error:" \
		"failed to source /usr/libexec/bash-base.bash" >&2
	exit 1
}

function cleanup() {
    log "cleanup called"
}

# replace the contents with your code
function main() {
    echo "this function is an example"
    echo "(un)comment lines below to test what happens on various failures"

    echo "main was called with args: '$*'"

    # make sure cleanup() is called at script exit
    bb_register_exit_hook mycleanuphook cleanup

    ## fatal error manually stops execution whereever called
    ## still you will get stack trace so you know where it happened
    fatal "kenny?"

    ## access to unset variables will trigger an error that is not catched
    ## by the ERR trap but by the EXIT trap. As a result the last line
    ## number of the stack trace will be incorrect (always 1)
    echo "$NoSuchVariableDefined"

    ## a command that fails: will trigger an error inside of main
    false

    ## this will sucessfully end function main, but since the return value
    ## is != 0 it will trigger an error where main was called
    return 123
}

# just call a function, never run code directly
main "$@"

# always use bb_quit to quit, it cares about traps and hooks
bb_quit
```
